@@ -1,18 +1,25 @@
 ;;; -*- Emacs-Lisp -*-
 ;;; YaTeX facilities for Emacs 19
-;;; (c )1994-1999 by HIROSE Yuuji.[yuuji@gentei.org]
-;;; Last modified Thu Apr 29 18:40:26 1999 on firestorm
-;;; $Id: yatex19.el,v 1.67 1999/07/15 04:58:48 yuuji Rel $
-
-;;; Ƃ肠 hilit19 gĂ鎞ɐFt悤ɂ
-;;; j[o[łɂ傲ɂł悤ɂB
-;;; Nj[o[gLaTeX\[X񂾂낤?
-;;; ܂KKB̕ɂƃRgB
-;;; ^ӂɂ邯ǁAhilit19.el Ή͌̕\{CB
-;;; ƂĂ邤 hilit19  obsolete ɂȂĂ܂c
+;;; (c )1994-2000 by HIROSE Yuuji.[yuuji@yatex.org]
+;;; Last modified Mon Dec 25 19:16:04 2000 on firestorm
+;;; $Id: yatex19.el,v 1.69 2000/12/25 10:19:50 yuuji Rel $
 
 ;(require 'yatex)
 
+(cond
+ (YaTeX-use-hilit19	(require 'hilit19))
+ (YaTeX-use-font-lock	(require 'font-lock)))
+
+(defvar YaTeX-use-highlighting (or YaTeX-use-font-lock YaTeX-use-hilit19)
+  "*Use highlighting buffer or not.")
+(defvar YaTeX-background-mode
+  (cond
+   ((boundp 'hilit-background-mode) hilit-background-mode)
+   ((boundp 'frame-background-mode) frame-background-mode)
+   ((fboundp 'get-frame-background-mode)
+    (get-frame-background-mode (selected-frame)))
+   (t nil)))
+
 (defvar YaTeX-mode-menu-map (make-sparse-keymap "YaTeX"))
 (defvar YaTeX-mode-menu-map-process (make-sparse-keymap "Process"))
 (define-key YaTeX-mode-map [menu-bar yatex]
@@ -222,6 +229,12 @@
 ;; j[ɔׂĂ͌\{CłĂ܂B
 ;; ĕ\ƂĂՂ񂾂B
 ;; ݂Ȃ hilit19.el g!
+;; ƂĂ邤 hilit19  obsolete ɂȂĂ܂c
+;; cƂƂŁAhilit19 p̃p^[ font-lock ɕϊ֐
+;; 쐬ĂȂƂ font-lock ɂΉ(2000N12)B
+;; Afont-lock ͎dlς₷͋CłƓ邩
+;; ǂ͕sBނi̎~܂hilit19gS
+;; ȂƂȂ̗͓ǂ߂ccB
 ;;
 ;; āA܂͑Ή {} sJsJ͈͂Ƃ悤Ȋ֐B
 ;;  hilit-LaTeX.el QlɂBłA section ^R}h
@@ -237,28 +250,38 @@
 	  (skip-chars-forward " \n\t*")
 	  (while (looking-at "\\[") (forward-list 1)) ;optionȂXLbv
 	  (skip-chars-forward " \n\t")
-	  (if (looking-at "{")		;{}n܂Ȃ炿Ƃsection^
-	      (cons m0
-		    (progn ;(skip-chars-backward "^{") (forward-char -2)
-		      (while (> argc 0)
-			(skip-chars-forward "^{")
-			(forward-list 1)
-			(setq argc (1- argc)))
-		      (point)))
-	    ;{}łȂȂ炽Ԃ \verb Ȃǂɂ_~[
-	    (cons m0 e0))))))
+	  (prog1
+	      (if (looking-at "{")	;{}n܂Ȃ炿Ƃsection^
+		  (cons m0
+			(condition-case err
+			    (progn
+			      ;;(skip-chars-backward "^{") (forward-char -2)
+			      (while (> argc 0)
+				(skip-chars-forward "^{")
+				(forward-list 1)
+				(setq argc (1- argc)))
+			      (point))
+			  (error m0)))
+			;{}łȂȂ炽Ԃ \verb Ȃǂɂ_~[
+		(cons m0 e0))
+	    ;;move to re-search end not to make font-lock confused
+	    (goto-char e0))))))
 
 (defun YaTeX-19-region-large-type (pattern)
   "Return list of large-type contents.
 Assumes PATTERN begins with `{'."
   (if (re-search-forward pattern nil t)
-      (let ((m0 (match-beginning 0)))
+      (let ((m0 (match-beginning 0)) (e0 (match-end 0))p)
 	(goto-char m0)
 	(skip-chars-forward "^ \t\n")
 	(skip-chars-forward " \t\n")
-	(cons (point)
-	      (progn (goto-char m0) (forward-list 1)
-		     (1- (point)))))))
+	(prog1
+	    (cons (setq p (point))
+		  (condition-case err
+		      (progn (goto-char m0) (forward-list 1) (1- (point)))
+		    (error (1+ p))))
+	  ;;move to re-search end not to make font-lock confused
+	  (goto-char e0)))))
 
 ;; ׂȂƂ % ̑O̕sJƂĂ܂悤Łc >hilit19
 ;; ̊֐͉ hilit-set-mode-patterns  "[^\\]\\(%\\).*$" 
@@ -324,7 +347,7 @@ Assumes PATTERN begins with `{'."
     ("\\\\begin{\\(eqn\\|equation\\|x?x?align\\|split\\|multline\\|gather\\)"
      "\\\\end{\\(eqn\\|equation\\|x?x?align\\|split\\|multline\\|gather\\).*}"
      formula)
-    ("[^\\$]\\($\\($[^$]*\\$\\|[^$]*\\)\\$\\)" 1 formula); '$...$' or '$$...$$'
+    ("\\([^\\$]\\|^\\)\\($\\($[^$]*\\$\\|[^$]*\\)\\$\\)" 2 formula); '$...$' or '$$...$$'
 
     ;; "wysiwyg" emphasis -- these don't work on nested expressions
     (YaTeX-19-region-large-type "{\\\\\\(em\\|it\\|sl\\)"  italic)
@@ -333,7 +356,7 @@ Assumes PATTERN begins with `{'."
     ;;;("\\\\begin{verbatim" "\\\\end{verbatim" tt)
 
     ("``" "''" string))
-"*Hiliting pattern alist for LaTeX text.")
+  "*Hiliting pattern alist for LaTeX text.")
 
 ;;(defvar YaTeX-hilit-pattern-adjustment-default nil)
 ;; ȂȂB
@@ -350,80 +373,126 @@ towards to lowest sectioning unit.  Numbers should be written in percentage.")
 (defvar YaTeX-sectioning-patterns-alist nil
   "Hilightening patterns for sectioning units.")
 (defvar YaTeX-hilit-singlecmd-face
-  '(slateblue2 aquamarine)
+  '("slateblue2" . "aquamarine")
   "*Hilightening face for maketitle type.  '(FaceForLight FaceForDark)")
 
 ;;; ZNVR}hA\x̍ɉĐF̔Zxς
 ;;; wiłȂƉ̂ȂɈႢȂ.
 ;;; Ĕn͍̎\xɉĐF𖾂邭قǂ̂?
+;;; ...ǂ炻łȂ炵Bł݂(2000/12)B
 ;(if (fboundp 'win32-color-values)
 ;    (fset 'x-color-values 'win32-color-values))
 
+(defun YaTeX-19-create-face (sym fgcolor &optional bgcolor)
+  "Create face named SYM with face of FGCOLOR/BGCOLOR."
+  (cond
+   ((and YaTeX-use-font-lock (fboundp 'defface))
+    (custom-declare-face
+     sym
+     (list
+      (list (list
+	     '(class color)
+	     ;(list 'background YaTeX-background-mode)
+	     )
+	    (delq nil
+		  (append
+		   (list ':foreground fgcolor)
+		   (if bgcolor
+		       (list ':background bgcolor))
+		   ))
+	    )
+      (list t (list ':bold t ':underline t))
+      )
+     (format "Font lock face for %s" sym)
+      ':group 'font-lock-faces)
+    (set sym sym)
+    sym)
+   ((and YaTeX-use-hilit19 (and (fboundp 'hilit-translate)))
+    (let ((face (intern (concat fgcolor "/" bgcolor))))
+      (hilit-translate sym face)
+      face))))
+
 (cond
- ((and (featurep 'hilit19) (fboundp 'x-color-values))
-  (let*((sectface
-	 (car (if (eq hilit-background-mode 'dark)
-		  (cdr YaTeX-hilit-sectioning-face)
-		YaTeX-hilit-sectioning-face)))
-	(sectcol (symbol-name sectface))
-	sect-pat-alist)
-    (if (string-match "/" sectcol)
-	(let ((fmin (nth 0 YaTeX-hilit-sectioning-attenuation-rate))
-	      (bmin (nth 1 YaTeX-hilit-sectioning-attenuation-rate))
-	      colorvalue fR fG fB bR bG bB pat fg bg level from face list lm)
-	  (require 'yatexsec)
-	  (setq fg (substring sectcol 0 (string-match "/" sectcol))
-		bg (substring sectcol (1+ (string-match "/" sectcol)))
-		colorvalue (x-color-values fg)
-		fR (/ (nth 0 colorvalue) 256)
-		fG (/ (nth 1 colorvalue) 256)
-		fB (/ (nth 2 colorvalue) 256)
-		colorvalue (x-color-values bg)
-		bR (/ (nth 0 colorvalue) 256)
-		bG (/ (nth 1 colorvalue) 256)
-		bB (/ (nth 2 colorvalue) 256)
-		lm YaTeX-sectioning-max-level
-		list YaTeX-sectioning-level)
-	  (while list
-	    (setq pat (concat YaTeX-ec-regexp (car (car list))
-			      ;"\\*?\\(\\[[^]]*\\]\\)?\\>" ;s͂ނƑʖ
-			      "\\>"
-			      )
-		  level (cdr (car list))
-		  fg (format "hex-%02x%02x%02x"
-			     (- fR (/ (* level fR fmin) lm 100))
-			     (- fG (/ (* level fG fmin) lm 100))
-			     (- fB (/ (* level fB fmin) lm 100)))
-		  bg (format "hex-%02x%02x%02x"
-			     (- bR (/ (* level bR bmin) lm 100))
-			     (- bG (/ (* level bG bmin) lm 100))
-			     (- bB (/ (* level bB bmin) lm 100)))
-		  from (intern (format "sectioning-%d" level))
-		  face (intern (concat fg "/" bg)))
-	    (hilit-translate from face)
-	    (setq sect-pat-alist
-		  (cons;;(list pat "}" face)
-		   (list 'YaTeX-19-region-section-type pat face)
-		   sect-pat-alist))
-	    (setq list (cdr list)))
-	  (setq YaTeX-sectioning-patterns-alist sect-pat-alist))))))
+ (YaTeX-use-highlighting
+  (cond
+   (window-system
+    (let*((sectface
+	   (car (if (eq YaTeX-background-mode 'dark)
+		    (cdr YaTeX-hilit-sectioning-face)
+		  YaTeX-hilit-sectioning-face)))
+	  (sectcol (symbol-name sectface))
+	  (fl YaTeX-use-font-lock)
+	  (form (if fl "#%02x%02x%02x" "hex-%02x%02x%02x"))
+	  sect-pat-alist)
+      (if (string-match "/" sectcol)
+	  (let ((fmin (nth 0 YaTeX-hilit-sectioning-attenuation-rate))
+		(bmin (nth 1 YaTeX-hilit-sectioning-attenuation-rate))
+		colorvalue fR fG fB bR bG bB pat fg bg level from face list lm)
+	    (require 'yatexsec)
+	    (setq fg (substring sectcol 0 (string-match "/" sectcol))
+		  bg (substring sectcol (1+ (string-match "/" sectcol)))
+		  colorvalue (x-color-values fg)
+		  fR (/ (nth 0 colorvalue) 256)
+		  fG (/ (nth 1 colorvalue) 256)
+		  fB (/ (nth 2 colorvalue) 256)
+		  colorvalue (x-color-values bg)
+		  bR (/ (nth 0 colorvalue) 256)
+		  bG (/ (nth 1 colorvalue) 256)
+		  bB (/ (nth 2 colorvalue) 256)
+		  lm YaTeX-sectioning-max-level
+		  list YaTeX-sectioning-level)
+	    (while list
+	      (setq pat (concat YaTeX-ec-regexp (car (car list))
+				;;"\\*?\\(\\[[^]]*\\]\\)?\\>" ;s͂ނƑʖ
+				"\\>"
+				)
+		    level (cdr (car list))
+		    fg (format form
+			       (- fR (/ (* level fR fmin) lm 100))
+			       (- fG (/ (* level fG fmin) lm 100))
+			       (- fB (/ (* level fB fmin) lm 100)))
+		    bg (format form
+			       (- bR (/ (* level bR bmin) lm 100))
+			       (- bG (/ (* level bG bmin) lm 100))
+			       (- bB (/ (* level bB bmin) lm 100)))
+		    from (intern (format "YaTeX-sectioning-%d" level))
+		    ;;face (intern (concat fg "/" bg))
+		    )
+	      (setq face (YaTeX-19-create-face from fg bg))
+	      (setq sect-pat-alist
+		    (cons;;(list pat "}" face)
+		     (list 'YaTeX-19-region-section-type pat face)
+		     sect-pat-alist))
+	      (setq list (cdr list)))
+	    (setq YaTeX-sectioning-patterns-alist sect-pat-alist)))))
+   (t					;not window-system
+    (setq YaTeX-sectioning-patterns-alist
+	  (list
+	   (list
+	    (concat YaTeX-ec-regexp
+		    "\\(\\(sub\\)*\\(section\\|paragraph\\)\\|chapter"
+		    "\\|part\\){[^}]*}")
+	    0
+	    'define)))))))
 
 ;; [Jȃ}Nǂݍ񂾌 redraw 
 ;; [J}N keyword ƂČ点(keyword܂ȁc)B
+(defvar hilit-patterns-alist nil)	;for absence of hilit19
+
 (defun YaTeX-19-collect-macros ()
   (cond
-   ((and (featurep 'hilit19) (fboundp 'hilit-translate))
+   (YaTeX-use-highlighting
     (let ((get-face
 	   (function
 	    (lambda (table)
 	      (cond
-	       ((eq hilit-background-mode 'light) (car table))
-	       ((eq hilit-background-mode 'dark) (car (cdr table)))
+	       ((eq YaTeX-background-mode 'light) (car table))
+	       ((eq YaTeX-background-mode 'dark) (cdr table))
 	       (t nil)))))
 	  sect single)
-      (hilit-translate
+      (YaTeX-19-create-face ;;hilit-translate
        ;;sectioning (funcall get-face YaTeX-hilit-sectioning-face)
-       macro (funcall get-face YaTeX-hilit-singlecmd-face))
+       'macro (funcall get-face YaTeX-hilit-singlecmd-face))
       (if (setq sect (append user-section-table tmp-section-table))
 	  (setq sect (concat "\\\\\\("
 			     (mapconcat
@@ -464,10 +533,52 @@ towards to lowest sectioning unit.  Numbers should be written in percentage.")
   (interactive "P")
   (YaTeX-19-collect-macros)
   (hilit-recenter arg))
-(if (fboundp 'hilit-recenter)		;Replace hilit-recenter with
-    (mapcar (function (lambda (key)	;YaTeX-hilit-recenter in yatex-mode
-			(define-key YaTeX-mode-map key 'YaTeX-hilit-recenter)))
-	    (where-is-internal 'hilit-recenter)))
+(defvar YaTeX-19-recenter-function
+  (cond
+   (YaTeX-use-hilit19	(cons 'YaTeX-hilit-recenter 'hilit-recenter))
+   (YaTeX-use-font-lock	(cons 'YaTeX-font-lock-recenter 'recenter))
+   (t nil)))
+
+(if YaTeX-19-recenter-function
+    (let ((k (where-is-internal (cdr YaTeX-19-recenter-function))))
+      (while k
+	(define-key YaTeX-mode-map (car k) (car YaTeX-19-recenter-function))
+	(setq k (cdr k)))))
+
+(defun YaTeX-font-lock-recenter (&optional arg)
+  (interactive "P")
+  (setq YaTeX-font-lock-keywords
+	(YaTeX-convert-pattern-hilit2fontlock
+	 (cdr (assq 'yatex-mode (YaTeX-19-collect-macros))))
+	font-lock-keywords nil)
+  ;(save-excursion
+   ; (font-lock-fontify-region (window-start) (window-end)))
+  (font-lock-mode -1)			;is stupid, but sure.
+  (font-lock-mode 1))
+
+(defvar YaTeX-font-lock-keywords nil
+  "Pattern-face alist of yahtml-mode for font-lock")
+
+(defun YaTeX-font-lock-set-default-keywords ()
+  (put 'yatex-mode 'font-lock-defaults
+       (list 'YaTeX-font-lock-keywords nil nil))
+  (setq YaTeX-font-lock-keywords
+	(YaTeX-convert-pattern-hilit2fontlock
+	 (cons nil
+	       (append YaTeX-sectioning-patterns-alist
+		       YaTeX-hilit-pattern-adjustment-private
+		       YaTeX-hilit-patterns-alist)))))
+
+(if YaTeX-use-font-lock
+    (progn
+      (if (and (boundp 'hilit-mode-enable-list) hilit-mode-enable-list)
+	  ;;for those who use both hilit19 and font-lock
+	  (if (eq (car hilit-mode-enable-list) 'not)
+	      (or (member 'yatex-mode hilit-mode-enable-list)
+		  (nconc hilit-mode-enable-list (list 'yatex-mode)))
+	    (setq hilit-mode-enable-list
+		  (delq 'yatex-mode hilit-mode-enable-list))))
+      (YaTeX-font-lock-set-default-keywords)))
 
 (defun YaTeX-switch-to-new-window ()
   (let ((c 0) (i 1) (free (make-string win:max-configs ? )))
@@ -513,8 +624,7 @@ WARNING, This code is not perfect."
       (switch-to-buffer b)
       (goto-char p))
      (t (switch-to-buffer-other-frame (buffer-name b))
-	(goto-char p))))
-)
+	(goto-char p)))))
 
 ;;; reverseVideo ɂ hilit-background-mode  'dark
 ;;; ɂĂl͐ȂǂÂȂ肷ČÂ炢ȂB
@@ -528,8 +638,19 @@ WARNING, This code is not perfect."
 (and YaTeX-emacs-19
      (not (featurep 'xemacs))
      (boundp 'byte-compile-current-file)
-     (if (and (boundp 'window-system) window-system)
-	 (require 'hilit19)
-       (error "Byte compile this file on window system! Not `-nw'!")))
+     byte-compile-current-file
+     (progn
+       (if YaTeX-emacs-20 (require 'font-lock))
+       (if (and (boundp 'window-system) window-system)
+	   (require 'hilit19)
+	 (error "Byte compile this file on window system! Not `-nw'!"))))
 
 (provide 'yatex19)
+
+
+; Local variables:
+; fill-prefix: ";;; "
+; paragraph-start: "^$\\|\\|;;;$"
+; paragraph-separate: "^$\\|\\|;;;$"
+; buffer-file-coding-system: sjis
+; End:
