@@ -1,12 +1,11 @@
 ;;; -*- Emacs-Lisp -*-
 ;;; YaTeX add-in function generator.
 ;;; yatexgen.el rev.5
-;;; (c )1991-1995 by HIROSE Yuuji.[yuuji@ae.keio.ac.jp]
-;;; Last modified Thu Feb 15 01:44:24 1996 on nsr
-;;; $Id: yatexgen.el,v 1.65 1997/12/16 13:34:01 yuuji Exp $
+;;; (c )1991-1995,1999,2000 by HIROSE Yuuji.[yuuji@yatex.org]
+;;; Last modified Mon Dec 25 19:17:30 2000 on firestorm
+;;; $Id: yatexgen.el,v 1.69 2000/12/25 10:19:50 yuuji Rel $
 
 (require 'yatex)
-(provide 'yatexgen)
 
 (defmacro YaTeX-setq (var japanese english)
   (list 'setq var
@@ -143,7 +142,7 @@ Send bug report to me."
   10 "Window height of YaTeX-generate-message-buffer")
 
 ;; Do you need learning for generated function?
-;; If you need, please tell me (yuuji@ae.keio.ac.jp)
+;; If you need, please tell me (yuuji@yatex.org)
 ;;(defvar YaTeX-generate-variables-for-learning nil)
 ;;(defvar YaTeX-generate-current-completion-table nil)
 
@@ -153,7 +152,7 @@ Send bug report to me."
 (defun YaTeX-generate ()
   "Genarate YaTeX add-in function with enquiry."
   (interactive)
-  (if (< (screen-height) (+ YaTeX-generate-message-height 10))
+  (if (< (YaTeX-screen-height) (+ YaTeX-generate-message-height 10))
       (error YaTeX-generate-narrow-message))
   (put 'YaTeX-generate 'disabled t)
   (save-window-excursion
@@ -181,7 +180,7 @@ Send bug report to me."
 	  (YaTeX-suppress-sparse-keymap map)
 	  ;;First get input form.
 	  (recursive-edit)
-	  (setq input (buffer-string)
+	  (setq input (YaTeX-minibuffer-string)
 		end (1- (length input)))
 	  (if (string= "" input) (error YaTeX-generate-abort-message))
 	  (YaTeX-generate-move-to-add-in-position)
@@ -201,7 +200,7 @@ Send bug report to me."
 	  (unwind-protect
 	      (recursive-edit)
 	    (fset 'self-insert-command (symbol-function 'si)))
-	  (setq output (buffer-string))
+	  (setq output (YaTeX-minibuffer-string))
 	  (cond ((string= "" output)	(error YaTeX-generate-abort-message))
 		((string= input output)	(error YaTeX-generate-same-message))
 		((< (length output) (length input))
@@ -357,7 +356,7 @@ Referencing variables in parent function YaTeX-generate-parse-add-in."
    ((eq YaTeX-current-completion-type 'begin)
     (goto-char (point-min))
     (skip-chars-forward "^{")
-    (setq env-name
+    (setq YaTeX-env-name
 	  (buffer-substring (1+ (point))
 			    (progn (skip-chars-forward "^}") (point))))
     (forward-char 1))
@@ -375,9 +374,9 @@ Referencing variables in parent function YaTeX-generate-parse-add-in."
   (concat
    "YaTeX:"
    (cond
-    ((eq YaTeX-current-completion-type 'begin) env-name)
-    ((eq YaTeX-current-completion-type 'section) section-name)
-    ((eq YaTeX-current-completion-type 'maketitle) single-command)))
+    ((eq YaTeX-current-completion-type 'begin) YaTeX-env-name)
+    ((eq YaTeX-current-completion-type 'section) YaTeX-section-name)
+    ((eq YaTeX-current-completion-type 'maketitle) YaTeX-single-command)))
 )
 
 (defun YaTeX-generate-lisp-quote (str)
@@ -591,13 +590,14 @@ Referencing variables in parent function YaTeX-generate-parse-add-in."
   (or command
       (setq command
 	    (completing-read
-	     (format "Making add-in function for (default %s): " section-name)
+	     (format
+	      "Making add-in function for (default %s): " YaTeX-section-name)
 	     (append
 	      section-table user-section-table tmp-section-table
 	      env-table     user-env-table     tmp-env-table
 	      singlecmd-table user-singlecmd-table tmp-singlecmd-table)
 	     nil nil)
-	    command (if (string= "" command) section-name command)))
+	    command (if (string= "" command) YaTeX-section-name command)))
   (message
    (cond
     (YaTeX-japan "(o)ǉ^? (a)^? (yatexadd.docQƂ̂) :")
@@ -609,3 +609,4 @@ Referencing variables in parent function YaTeX-generate-parse-add-in."
       command
       (string-to-int (read-string "How many arguments?: ")))) nil)
 )
+(provide 'yatexgen)
