@@ -1,21 +1,32 @@
 ;;; -*- Emacs-Lisp -*-
 ;;; YaTeX and yahtml common libraries, general functions and definitions
 ;;; yatexlib.el
-;;; (c )1994-1999 by HIROSE Yuuji.[yuuji@gentei.org]
-;;; Last modified Tue May  4 10:25:55 1999 on firestorm
-;;; $Id: yatexlib.el,v 1.67 1999/07/15 04:58:48 yuuji Rel $
+;;; (c )1994-2000 by HIROSE Yuuji.[yuuji@yatex.org]
+;;; Last modified Mon Dec 25 18:51:11 2000 on firestorm
+;;; $Id: yatexlib.el,v 1.69 2000/12/25 10:19:50 yuuji Rel $
 
 ;; General variables
 (defvar YaTeX-dos (memq system-type '(ms-dos windows-nt OS/2)))
 (defvar YaTeX-emacs-19 (>= (string-to-int emacs-version) 19))
 (defvar YaTeX-emacs-20 (>= (string-to-int emacs-version) 20))
+(defvar YaTeX-emacs-21 (>= (string-to-int emacs-version) 21))
 (defvar YaTeX-user-completion-table
   (if YaTeX-dos "~/_yatexrc" "~/.yatexrc")
   "*Default filename in which user completion table is saved.")
 
+(defvar YaTeX-display-color-p
+  (or (and (fboundp 'display-color-p) (display-color-p))
+      (and (fboundp 'device-class)
+	   (eq 'color (device-class (selected-device))))
+      window-system)  ; falls down lazy check..
+  "Current display's capability of expressing colors.")
+
 (defvar YaTeX-japan (or (boundp 'NEMACS) (boundp 'MULE) YaTeX-emacs-20)
   "Whether yatex mode is running on Japanese environment or not.")
 
+;; autoload from yahtml.el
+(autoload 'yahtml-inner-environment-but "yahtml" "yahtml internal func." t)
+
 (defvar YaTeX-kanji-code-alist
   (cond
    ((boundp '*junet*)
@@ -52,6 +63,11 @@ If nil, use default pop-to-buffer.")
   "*If you are nervous about maintenance of yatexrc, set this value to T.
 And you will have the local dictionary.")
 
+(defvar YaTeX-use-italic-bold (string< "20" emacs-version)
+  "*Non-nil tries to find italic/bold fontset.
+This variable is effective when font-lock is used.
+\it, \bf ł̓{ꂪɂȂĂ܂ꍇ͂nilɂĉB")
+
 ;----------- work variables ----------------------------------------
 (defvar YaTeX-typesetting-mode-map nil
   "Keymap used in YaTeX typesetting buffer")
@@ -292,6 +308,7 @@ Optional sixth argument FUNC changes search-function."
 		 (and (eq major-mode 'yatex-mode)
 		      (YaTeX-in-verb-p (match-beginning 0)))
 		 (save-excursion
+		   (goto-char (match-beginning 0))
 		   (beginning-of-line)
 		   (re-search-forward cmntrx (match-beginning 0) t)))))
     (store-match-data md)
@@ -318,7 +335,14 @@ See also YaTeX-search-active-forward."
   "Switch to buffer if buffer exists, find file if not.
 Optional second arg SETBUF t make use set-buffer instead of switch-to-buffer."
   (interactive "Fswitch to file: ")
-  (if (bufferp file) (setq file (buffer-file-name file)))
+  (if (bufferp file)
+      (setq file (buffer-file-name file))
+    (and (string-match "^[^/].*/" file)
+	 (eq major-mode 'yatex-mode)
+	 YaTeX-search-file-from-top-directory
+	 (save-excursion
+	   (YaTeX-visit-main t)
+	   (setq file (expand-file-name file)))))
   (let (buf (hilit-auto-highlight (not setbuf)))
     (cond
      ((setq buf (get-file-buffer file))
@@ -337,6 +361,13 @@ Optional second arg SETBUF t make use set-buffer instead of switch-to-buffer."
 (defun YaTeX-switch-to-buffer-other-window (file)
   "Switch to buffer if buffer exists, find file if not."
   (interactive "Fswitch to file: ")
+  (and (eq major-mode 'yatex-mode)
+       (stringp file)
+       (string-match "^[^/].*/" file)
+       YaTeX-search-file-from-top-directory
+       (save-excursion
+	 (YaTeX-visit-main t)
+	 (setq file (expand-file-name file))))
   (if (bufferp file) (setq file (buffer-file-name file)))
   (cond
    ((get-file-buffer file)
@@ -347,13 +378,24 @@ Optional second arg SETBUF t make use set-buffer instead of switch-to-buffer."
    (t (message "%s was not found in this directory." file)
       nil)))
 
+(defun YaTeX-get-file-buffer (file)
+  "Return the FILE's buffer.
+Base directory is that of main file or current directory."
+  (let (dir main (cdir default-directory))
+    (or (and (eq major-mode 'yatex-mode)
+	     YaTeX-search-file-from-top-directory
+	     (save-excursion
+	       (YaTeX-visit-main t)
+	       (get-file-buffer file)))
+	(get-file-buffer file))))
+
 (defun YaTeX-replace-format-sub (string format repl)
   (let ((beg (or (string-match (concat "^\\(%" format "\\)") string)
 		 (string-match (concat "[^%]\\(%" format "\\)") string)))
 	(len (length format)))
     (if (null beg) string ;no conversion
       (concat
-       (substring string 0 (match-beginning 1)) repl
+       (substring string 0 (match-beginning 1)) (or repl "")
        (substring string (match-end 1))))))
 
 ;;;###autoload
@@ -366,6 +408,15 @@ function `format' does.  FORMAT does not contain `%'"
       (setq ans string))
     string))
 
+;;;###autoload
+(defun YaTeX-replace-formats (string replace-list)
+  (let ((list replace-list))
+    (while list
+      (setq string (YaTeX-replace-format
+		    string (car (car list)) (cdr (car list)))
+	    list (cdr list)))
+    string))
+
 ;;;###autoload
 (defun YaTeX-replace-format-args (string &rest args)
   "Translate the argument mark #1, #2, ... #n in the STRING into the
@@ -480,6 +531,19 @@ Otherwise split window conventionally."
 	  (setq wlist (cons win wlist))))
     wlist))
 
+(if YaTeX-emacs-21
+    ;; Emacs-21's next-window returns other frame's window even if called
+    ;; with argument ALL-FRAMES nil, when called from minibuffer context.
+    ;; Therefore, check frame identity here.
+    (defun YaTeX-window-list ()
+      (let*((curw (selected-window)) (win curw) (wlist (list curw))
+	    (curf (window-frame curw)))
+	(while (and (not (eq curw (setq win (next-window win))))
+		    (eq curf (window-frame win)))
+	  (or (eq win (minibuffer-window))
+	      (setq wlist (cons win wlist))))
+	wlist)))
+
 ;;;###autoload
 (defun substitute-all-key-definition (olddef newdef keymap)
   "Replace recursively OLDDEF with NEWDEF for any keys in KEYMAP now
@@ -507,53 +571,57 @@ the last field separated by those delimiters.
   If the symbol 'quick is bound and is 't, when the try-completion results
 in t, exit minibuffer immediately."
   (interactive)
-  (let ((md (match-data)) beg word compl
-	(quick (and (boundp 'quick) (eq quick t)))
-	(displist			;function to display completion-list
-	 (function
-	  (lambda ()
-	    (with-output-to-temp-buffer "*Completions*"
-	      (display-completion-list
-	       (all-completions word minibuffer-completion-table)))))))
-    (setq beg (if (and (boundp 'delim) (stringp delim))
-		  (save-excursion
-		    (skip-chars-backward (concat "^" delim))
-		    (point))
-		(point-min))
-	  word (buffer-substring beg (point-max))
-	  compl (try-completion word minibuffer-completion-table))
-    (cond
-     ((eq compl t)
-      (if quick (exit-minibuffer)
-	(let ((p (point)) (max (point-max)))
-	  (unwind-protect
-	      (progn
-		(goto-char max)
-		(insert " [Sole completion]")
-		(goto-char p)
-		(sit-for 1))
-	    (delete-region max (point-max))
-	    (goto-char p)))))
-     ((eq compl nil)
-      (ding)
-      (save-excursion
-	(let (p)
-	  (unwind-protect
-	      (progn
-		(goto-char (setq p (point-max)))
-		(insert " [No match]")
-		(goto-char p)
-		(sit-for 2))
-	    (delete-region p (point-max))))))
-     ((string= compl word)
-      (funcall displist))
-     (t (delete-region beg (point-max))
-	(insert compl)
-	(if quick
-	    (if (eq (try-completion compl minibuffer-completion-table) t)
-		(exit-minibuffer)
-	      (funcall displist)))))
-    (store-match-data md)))
+  (save-restriction
+    (narrow-to-region
+     (if (fboundp 'field-beginning) (field-beginning (point-max)) (point-min))
+     (point-max))
+    (let ((md (match-data)) beg word compl
+	  (quick (and (boundp 'quick) (eq quick t)))
+	  (displist ;function to display completion-list
+	   (function
+	    (lambda ()
+	      (with-output-to-temp-buffer "*Completions*"
+		(display-completion-list
+		 (all-completions word minibuffer-completion-table)))))))
+      (setq beg (if (and (boundp 'delim) (stringp delim))
+		    (save-excursion
+		      (skip-chars-backward (concat "^" delim))
+		      (point))
+		  (point-min))
+	    word (buffer-substring beg (point-max))
+	    compl (try-completion word minibuffer-completion-table))
+      (cond
+       ((eq compl t)
+	(if quick (exit-minibuffer)
+	  (let ((p (point)) (max (point-max)))
+	    (unwind-protect
+		(progn
+		  (goto-char max)
+		  (insert " [Sole completion]")
+		  (goto-char p)
+		  (sit-for 1))
+	      (delete-region max (point-max))
+	      (goto-char p)))))
+       ((eq compl nil)
+	(ding)
+	(save-excursion
+	  (let (p)
+	    (unwind-protect
+		(progn
+		  (goto-char (setq p (point-max)))
+		  (insert " [No match]")
+		  (goto-char p)
+		  (sit-for 2))
+	      (delete-region p (point-max))))))
+       ((string= compl word)
+	(funcall displist))
+       (t (delete-region beg (point-max))
+	  (insert compl)
+	  (if quick
+	      (if (eq (try-completion compl minibuffer-completion-table) t)
+		  (exit-minibuffer)
+		(funcall displist)))))
+      (store-match-data md))))
 
 (defun YaTeX-minibuffer-quick-complete ()
   "Set 'quick to 't and call YaTeX-minibuffer-complete.
@@ -643,13 +711,49 @@ If no such window exist, switch to buffer BUFFER."
 (fset 'YaTeX-rassoc
       (if (and nil (fboundp 'rassoc) (subrp (symbol-function 'rassoc)))
 	  (symbol-function 'rassoc)
-	(lambda (key list)
-	  (let ((l list))
-	    (catch 'found
-	      (while l
-		(if (equal key (cdr (car l)))
-		    (throw 'found (car l)))
-		(setq l (cdr l))))))))
+	(function
+	 (lambda (key list)
+	   (let ((l list))
+	     (catch 'found
+	       (while l
+		 (if (equal key (cdr (car l)))
+		     (throw 'found (car l)))
+		 (setq l (cdr l)))))))))
+
+(defun YaTeX-insert-file-contents (file visit &optional beg end)
+  (cond
+   ((and (string< "19" emacs-version) (not (featurep 'xemacs)))
+    (insert-file-contents file visit beg end))
+   ((string-match "unix" (symbol-name system-type))
+    (let ((default-process-coding-system
+	    (and (boundp '*noconv*) (list '*noconv*)))
+	  (file-coding-system (and (boundp '*noconv*) '*noconv*))
+	  kanji-fileio-code
+	  (default-process-kanji-code 0))
+      (call-process shell-file-name file (current-buffer) nil
+		    (or (and (boundp 'shell-command-option)
+			     shell-command-option)
+			"-c")
+		    (format "head -c %d | tail -c +%d" end beg))))
+    (t (insert-file-contents file))))
+
+(defun YaTeX-split-string (str &optional sep null)
+  "Split string STR by every occurrence of SEP(regexp).
+If the optional second argument SEP is nil, it defaults to \"[ \f\t\n\r\v]+\".
+Do not include null string by default.  Non-nil for optional third argument
+NULL includes null string in a list."
+  (let ((sep (or sep "[ \f\t\n\r\v]+"))
+	list m)
+    (while str
+      (if (setq m (string-match sep str))
+	  (progn
+	    (if (or (> m 0) null)
+		(setq list (cons (substring str 0 m) list)))
+	    (setq str (substring str (match-end 0))))
+	(if (or null (string< "" str))
+	    (setq list (cons str list)))
+	(setq str nil)))
+    (nreverse list)))
 
 ;;;
 ;; Interface function for windows.el
@@ -752,6 +856,62 @@ of 'YaTeX-inner-environment, which can be referred by
 		   (YaTeX-replace-format-args YaTeX-struct-begin env "" "")
 		   (count-lines (point-min) (point))))))))
 
+(defun YaTeX-beginning-of-environment (&optional limit-search-bound end)
+  "Goto the beginning of the current environment.
+Optional argument LIMIT-SEARCH-BOUND non-nil limits the search bound to
+most recent sectioning command.  Non-nil for optional third argument END
+goes to end of environment."
+  (interactive)
+  (let ((op (point)))
+    (if (YaTeX-inner-environment limit-search-bound)
+	(progn
+	  (goto-char (get 'YaTeX-inner-environment 'point))
+	  (and end (YaTeX-goto-corresponding-environment))
+	  (if (interactive-p) (push-mark op))
+	  t))))
+
+(defun YaTeX-end-of-environment (&optional limit-search-bound)
+  "Goto the end of the current environment.
+Optional argument LIMIT-SEARCH-BOUND non-nil limits the search bound
+to most recent sectioning command."
+  (interactive)
+  (YaTeX-beginning-of-environment limit-search-bound t))
+
+(defun YaTeX-mark-environment ()
+  "Mark current position and move point to end of environment."
+  (interactive)
+  (let ((curp (point)))
+    (if (and (YaTeX-on-begin-end-p) (match-beginning 1)) ;if on \\begin
+	(forward-line 1)
+      (beginning-of-line))
+    (if (not (YaTeX-end-of-environment))   ;arg1 turns to match-beginning 1
+	(progn
+	  (goto-char curp)
+	  (error "Cannot found the end of current environment."))
+      (YaTeX-goto-corresponding-environment)
+      (beginning-of-line)		;for confirmation
+      (if (< curp (point))
+	  (progn
+	    (message "Mark this environment?(y or n): ")
+	    (if (= (read-char) ?y) nil
+	      (goto-char curp)
+	      (error "Abort.  Please call again at more proper position."))))
+      (set-mark-command nil)
+      (YaTeX-goto-corresponding-environment)
+      (end-of-line)
+      (if (eobp) nil (forward-char 1)))))
+
+(defun YaTeX-kill-buffer (buffer)
+  "Make effort to show parent buffer after kill."
+  (interactive "bKill buffer: ")
+  (or (get-buffer buffer)
+      (error "No such buffer %s" buffer))
+  (let ((pf YaTeX-parent-file))
+    (kill-buffer buffer)
+    (and pf
+	 (get-file-buffer pf)
+	 (switch-to-buffer (get-file-buffer pf)))))
+
 ;;;VER2
 (defun YaTeX-insert-struc (what env)
   (cond
@@ -802,6 +962,216 @@ See yatex19.el for example."
 	(define-key (symbol-value keymap) (vector (car bind)) (cdr bind))))
      bindlist))))
 
+;;;
+;; Emacs 21 compensational wrapper
+;;;
+(defun YaTeX-minibuffer-begin ()
+ (if (fboundp 'field-beginning)
+     (field-beginning (point-max))
+   (point-min)))
+
+(defun YaTeX-minibuffer-end ()
+ (if (fboundp 'field-end)
+     (field-end (point-max))
+   (point-max)))
+
+(defun YaTeX-minibuffer-string ()
+  (buffer-substring (YaTeX-minibuffer-begin) (YaTeX-minibuffer-end)))
+
+(defun YaTeX-minibuffer-erase ()
+  (if (eq (selected-window) (minibuffer-window))
+      (if (fboundp 'delete-field) (delete-field) (erase-buffer))))
+
+;;;
+;; hilit19 vs. font-lock
+;;;
+(defun YaTeX-convert-pattern-hilit2fontlock (h19pa)
+  "Convert hilit19's H19PA patterns alist to font-lock's one.
+This function is a makeshift for YaTeX and yahtml."
+  (let ((ignorecase (not (null (car h19pa))))
+	(palist (cdr h19pa))
+	flpa i newface
+	(mapping
+	 '((bold . YaTeX-font-lock-bold-face)
+	   (italic . YaTeX-font-lock-italic-face)
+	   (define . font-lock-function-name-face)
+	   (keyword . font-lock-keyword-face)
+	   (decl . YaTeX-font-lock-declaration-face)
+	   (label . YaTeX-font-lock-label-face)
+	   (crossref . YaTeX-font-lock-crossref-face)
+	   (include . YaTeX-font-lock-include-face)
+	   (formula . YaTeX-font-lock-formula-face)
+	   (string . ignore) (comment . ignore)
+	   )))
+    (while (setq i (car palist))
+      (setq newface (nth 2 i)
+	    newface (or (cdr (assq newface mapping)) newface))
+      (cond
+       ((eq newface 'ignore) nil)	;no translation
+       ((stringp (car i))		;hiliting by regexp
+	(setq flpa
+	      (cons
+	       (if (numberp (car (cdr i)))
+		   (list (car i)	;regexp
+			 (car (cdr i))	;matching group number
+			 newface t) ;'keep)	;keep is hilit19 taste
+		 (list
+		  (concat
+		   (car i)		;original regexp and..
+		   ;;"[^"
+		   ;;(regexp-quote (substring (car (cdr i)) 0 1))
+		   ;;"]+" ;for shortest match
+		   ".*"
+		   (car (cdr i)))
+		  0 (list 'quote newface) t)) ;;'keep))
+	       flpa)))
+       ((and (symbolp (car i)) (fboundp (car i)))
+	(setq flpa
+	      (cons
+	       (list (car (cdr i))	;regexp
+		     (list
+		      (list
+		       'lambda (list 'dummy)
+		       '(goto-char (match-beginning 0))
+		       '(remove-text-properties
+			 (point) (1+ (point))
+			 '(face nil font-lock-multiline nil))
+		       (list
+			'let (list '(e (match-end 0))
+				   (list 'm (list (car i) (car (cdr i)))))
+			(list
+			 'if 'm
+			 (list
+			  'YaTeX-font-lock-fillin
+			  (list 'car 'm)
+			  (list 'cdr 'm)
+			  (list 'quote 'face)
+			  (list 'quote 'font-lock)
+			  (list 'quote newface))
+			 '(goto-char e)
+			 ))
+		       nil)		;retun nil to cheat font-lock
+		      nil nil))		;pre-match, post-match both nil
+	       flpa))))
+      (setq palist (cdr palist)));while
+    (if (featurep 'xemacsp)
+	(nreverse flpa)
+      flpa)))
+
+(cond
+ ((featurep 'font-lock)
+  ;; In each defface, '(class static-color) is for Emacs-21 -nw
+  ;; '(class tty) is for XEmacs-21 -nw
+  (defface YaTeX-font-lock-label-face
+    '((((class static-color)) (:foreground "yellow" :underline t))
+      (((type tty)) (:foreground "yellow" :underline t))
+      (((class color) (background dark)) (:foreground "pink" :underline t))
+      (((class color) (background light)) (:foreground "red" :underline t))
+      (t (:bold t :underline t)))
+    "Font Lock mode face used to highlight labels."
+    :group 'font-lock-faces)
+  (defvar YaTeX-font-lock-label-face 'YaTeX-font-lock-label-face)
+
+  (defface YaTeX-font-lock-declaration-face
+    '((((class color) (background dark)) (:foreground "cyan"))
+      (((class color) (background light)) (:foreground "RoyalBlue"))
+      (t (:bold t :underline t)))
+    "Font Lock mode face used to highlight some declarations."
+    :group 'font-lock-faces)
+  (defvar YaTeX-font-lock-declaration-face 'YaTeX-font-lock-declaration-face)
+
+  (defface YaTeX-font-lock-include-face
+    '((((class color) (background dark)) (:foreground "Plum1"))
+      (((class color) (background light)) (:foreground "purple"))
+      (t (:bold t :underline t)))
+    "Font Lock mode face used to highlight expression for including."
+    :group 'font-lock-faces)
+  (defvar YaTeX-font-lock-include-face 'YaTeX-font-lock-include-face)
+
+  (defface YaTeX-font-lock-formula-face
+    '((((class static-color)) (:bold t))
+      (((type tty)) (:bold t))
+      (((class color) (background dark)) (:foreground "khaki" :bold t))
+      (((class color) (background light)) (:foreground "Goldenrod"))
+      (t (:bold t :underline t)))
+    "Font Lock mode face used to highlight formula."
+    :group 'font-lock-faces)
+  (defvar YaTeX-font-lock-formula-face 'YaTeX-font-lock-formula-face)
+
+  (defface YaTeX-font-lock-crossref-face
+    '((((class color) (background dark)) (:foreground "lightgoldenrod"))
+      (((class color) (background light)) (:foreground "DarkGoldenrod"))
+      (t (:bold t :underline t)))
+    "Font Lock mode face used to highlight cress references."
+    :group 'font-lock-faces)
+  (defvar YaTeX-font-lock-crossref-face 'YaTeX-font-lock-crossref-face)
+
+  (defface YaTeX-font-lock-bold-face
+    '((t (:bold t)))
+    "Font Lock mode face used to express bold itself."
+    :group 'font-lock-faces)
+  (defvar YaTeX-font-lock-bold-face 'YaTeX-font-lock-bold-face)
+
+  (defface YaTeX-font-lock-italic-face
+    '((t (:italic t)))
+    "Font Lock mode face used to express italic itself."
+    :group 'font-lock-faces)
+  (defvar YaTeX-font-lock-italic-face 'YaTeX-font-lock-italic-face)
+
+  ;; Make sure the 'YaTeX-font-lock-{italic,bold}-face is bound with
+  ;; italic/bold fontsets
+  (if (and (fboundp 'fontset-list) YaTeX-use-italic-bold)
+      (let ((flist (fontset-list)) fnt italic bold)
+	(while flist
+	  (setq fnt (car flist))
+	  (condition-case err
+	      (cond
+	       ((and (string-match "-medium-i-" fnt) (null italic))
+		(set-face-font 'YaTeX-font-lock-italic-face (setq italic fnt)))
+	       ((and (string-match "-bold-r-" fnt) (null bold))
+		(set-face-font 'YaTeX-font-lock-bold-face (setq bold fnt))))
+	    (error nil))
+	  (setq flist (cdr flist)))))
+
+  ;;Borrowed from XEmacs's font-lock.el
+  (defsubst YaTeX-font-lock-fillin (start end setprop markprop value &optional object)
+    "Fill in one property of the text from START to END.
+Arguments PROP and VALUE specify the property and value to put where none are
+already in place.  Therefore existing property values are not overwritten.
+Optional argument OBJECT is the string or buffer containing the text."
+    (let ((start (text-property-any start end markprop nil object)) next
+	  (putfunc (if (fboundp 'put-nonduplicable-text-property)
+		       'put-nonduplicable-text-property
+		     'put-text-property)))
+      (if (eq putfunc 'put-text-property)
+	  (setq markprop setprop))
+      (while start
+	(setq next (next-single-property-change start markprop object end))
+	(funcall putfunc start next setprop value object)
+	(funcall putfunc start next markprop value object)
+	(setq start (text-property-any next end markprop nil object)))))
+
+  (defun YaTeX-warning-font-lock (mode)
+    (let ((sw (selected-window)))
+      ;;(pop-to-buffer (format " *%s warning*" mode))
+      ;;(erase-buffer)
+      (momentary-string-display
+      (cond
+       (YaTeX-japan
+	(concat mode " ́A font-lock ɑΉ܂B\n"
+		"~/.emacs Ȃǂɂ\n"
+		"\t(put 'yatex-mode 'font-lock-keywords 'tex-mode)\n"
+		"\t(put 'yahtml-mode 'font-lock-keywords 'html-mode)\n"
+		"Ȃǂ̊Ԃɍ킹̋Lq͂͂svłB"))
+       (t
+	(concat mode " now supports the font-lock by itself.\n"
+		"So you can remove the descriptions such as\n"
+		"\t(put 'yatex-mode 'font-lock-keywords 'tex-mode)\n"
+		"\t(put 'yahtml-mode 'font-lock-keywords 'html-mode)\n"
+		"in your ~/.emacs file.  Thank you."))) (point))
+      (select-window sw)))
+  ))
+
 
 ;;;
 ;; Functions for the Installation time
@@ -817,5 +1187,10 @@ See yatex19.el for example."
 	(mapcar 'byte-compile-file command-line-args-left)
 	(kill-emacs))))
 
-	
 (provide 'yatexlib)
+; Local variables:
+; fill-prefix: ";;; "
+; paragraph-start: "^$\\|\\|;;;$"
+; paragraph-separate: "^$\\|\\|;;;$"
+; buffer-file-coding-system: sjis
+; End:
