@@ -1,9 +1,9 @@
 ;;; -*- Emacs-Lisp -*-
 ;;; YaTeX math-mode-specific functions.
 ;;; yatexmth.el
-;;; (c )1993-1999 by HIROSE Yuuji [yuuji@gentei.org]
-;;; Last modified Thu Apr 29 17:39:10 1999 on firestorm
-;;; $Id: yatexmth.el,v 1.67 1999/07/15 04:58:48 yuuji Rel $
+;;; (c)1993-2000 by HIROSE Yuuji [yuuji@yatex.org]
+;;; Last modified Tue Dec 19 11:03:51 2000 on buell
+;;; $Id: yatexmth.el,v 1.69 2000/12/25 10:19:50 yuuji Rel $
 
 ;;; [Customization guide]
 ;;;
@@ -406,7 +406,7 @@
 (defmacro YaTeX-math-japanese-sign (list)
   (list 'nth 1 list))
 
-(defvar YaTeX-math-cmd-regexp (concat (regexp-quote YaTeX-ec) "[A-z]"))
+(defvar YaTeX-math-cmd-regexp (concat (regexp-quote YaTeX-ec) "[A-z|]"))
 (defvar YaTeX-math-verbatim-environments
   '("alltt")
   "*List of environments in which LaTeX math mode is disabled.
@@ -551,7 +551,7 @@ This value is appended with YaTeX-verbatim-environments.")
 (defun YaTeX-math-show-menu (match-str)
   (save-window-excursion
     (YaTeX-showup-buffer YaTeX-math-menu-buffer nil t)
-    (let ((maxcols (max 1 (/ (screen-width) YaTeX-math-sign-width)))
+    (let ((maxcols (max 1 (/ (YaTeX-screen-width) YaTeX-math-sign-width)))
 	  (case-fold-search nil)
 	  (cols 0) (list alist) command)
       (erase-buffer)
@@ -560,7 +560,7 @@ This value is appended with YaTeX-verbatim-environments.")
       (insert YaTeX-math-indicator "\t")
       (insert YaTeX-math-indicator)
       (newline 1)
-      (insert-char ?- (1- (screen-width)))
+      (insert-char ?- (1- (YaTeX-screen-width)))
       (newline 1)
       (while list
 	(if (string-match match-str (car (car list)))
