@@ -1,9 +1,9 @@
 ;;; -*- Emacs-Lisp -*-
 ;;; YaTeX process handler.
 ;;; yatexprc.el
-;;; (c )1993-1999 by HIROSE Yuuji.[yuuji@gentei.org]
-;;; Last modified Tue Jul 13 13:47:46 1999 on firestorm
-;;; $Id: yatexprc.el,v 1.67 1999/07/15 04:58:48 yuuji Rel $
+;;; (c )1993-2000 by HIROSE Yuuji.[yuuji@yatex.org]
+;;; Last modified Mon Dec 25 19:18:19 2000 on firestorm
+;;; $Id: yatexprc.el,v 1.69 2000/12/25 10:19:50 yuuji Rel $
 
 ;(require 'yatex)
 (require 'yatexlib)
@@ -363,6 +363,40 @@ PROC should be process identifier."
 	  "system" buffer shell-file-name YaTeX-shell-command-option command)
 	 (get-buffer buffer))))))
 
+(defvar YaTeX-default-paper-type "a4"
+  "*Default paper type.")
+(defconst YaTeX-paper-type-alist
+  '(("a4paper" . "a4")
+    ("a5paper" . "a5")
+    ("b4paper" . "b4")
+    ("b5paper" . "b5"))
+  "Holds map of options and paper types.")
+(defconst YaTeX-dvips-paper-option-alist
+  '(("a4" . "-t a4")
+    ("a5" . "-t a5")
+    ("b4" . "-t b4")
+    ("b5" . "-t b5")
+    ("a4r" . "-t landscape"); Can't specify options, `-t a4' and `-t landscape', at the same time.
+    ("a5r" . "-t landscape")
+    ("b4r" . "-t landscape")
+    ("b5r" . "-t landscape"))
+  "Holds map of dvips options and paper types.")
+(defun YaTeX-get-paper-type ()
+  "Search options in header and return a paper type, such as \"a4\", \"a4r\", etc."
+  (save-excursion
+    (goto-char (point-min))
+    (if (re-search-forward
+	 "^[ \t]*\\\\document\\(style\\|class\\)[ \t]*\\[\\([^]]*\\)\\]" nil t)
+	(let ((opts (YaTeX-split-string (YaTeX-match-string 2) "[ \t]*,[ \t]*")))
+	  (concat
+	   (catch 'found-paper
+	     (mapcar (lambda (pair)
+		       (if (member (car pair) opts)
+			   (throw 'found-paper (cdr pair))))
+		     YaTeX-paper-type-alist)
+	     YaTeX-default-paper-type)
+	   (if (member "landscape" opts) "r" ""))))))
+
 (defvar YaTeX-preview-command-history nil
   "Holds minibuffer history of preview command.")
 (put 'YaTeX-preview-command-history 'no-default t)
@@ -375,7 +409,9 @@ PROC should be process identifier."
    (list
     (read-string-with-history
      "Preview command: "
-     (or (YaTeX-get-builtin "PREVIEW") dvi2-command)
+     (YaTeX-replace-format
+      (or (YaTeX-get-builtin "PREVIEW") dvi2-command)
+      "p" (concat "-paper " (YaTeX-get-paper-type)))
      'YaTeX-preview-command-history)
     (read-string-with-history
      "Preview file[.dvi]: "
@@ -479,7 +515,7 @@ error or warning lines in reverse order."
 	   (buffer-substring
 	    (point)
 	    (progn (skip-chars-forward "0-9" (match-end 0)) (point))))
-	  error-buffer (YaTeX-get-error-file cur-buf))
+	  error-buffer (expand-file-name (YaTeX-get-error-file cur-buf)))
     (if (or (null error-line) (equal 0 error-line))
 	(error "Can't detect error position."))
     (YaTeX-set-virtual-error-position 'error-buffer 'error-line)
@@ -517,7 +553,8 @@ error or warning lines in reverse order."
       (goto-char (match-beginning 0))
       (setq error-line (string-to-int
 			(buffer-substring (match-beginning 1) (match-end 1)))
-	    error-file (YaTeX-get-error-file YaTeX-current-TeX-buffer))
+	    error-file (expand-file-name
+			(YaTeX-get-error-file YaTeX-current-TeX-buffer)))
       (YaTeX-set-virtual-error-position 'error-file 'error-line)
       (setq error-buf (YaTeX-switch-to-buffer error-file t)))
       (if (null error-buf)
@@ -676,6 +713,11 @@ page range description."
 	  ""
 	(YaTeX-replace-format dviprint-to-format "e" to)))
      )
+    (setq
+     cmd
+     (YaTeX-replace-format
+      cmd "p"
+      (cdr (assoc (YaTeX-get-paper-type) YaTeX-dvips-paper-option-alist))))
     (setq cmd
 	  (read-string-with-history
 	   "Edit command line: "
@@ -710,7 +752,9 @@ page range description."
    (YaTeX-parent-file
     (eq (get-file-buffer YaTeX-parent-file) (current-buffer)))
    ((YaTeX-get-builtin "!")
-    (string-match (YaTeX-guess-parent (YaTeX-get-builtin "!")) (buffer-name)))
+    (string-match
+     (concat "^" (YaTeX-guess-parent (YaTeX-get-builtin "!")))
+     (buffer-name)))
    (t
     (save-excursion
       (let ((latex-main-id
@@ -724,7 +768,11 @@ Use set-buffer instead of switch-to-buffer if the optional second argument
 SETBUF is t(Use it only from Emacs-Lisp program)."
   (interactive "P")
   (if (and (interactive-p) setbuf) (setq YaTeX-parent-file nil))
-  (let (b-in main-file)
+  (let ((ff (function (lambda (f)
+			(if setbuf (set-buffer (find-file-noselect f))
+			  (find-file f)))))
+	b-in main-file YaTeX-create-file-prefix-g
+	(hilit-auto-highlight (not setbuf)))
     (if (setq b-in (YaTeX-get-builtin "!"))
 	(setq main-file (YaTeX-guess-parent b-in)))
     (if YaTeX-parent-file
@@ -733,18 +781,30 @@ SETBUF is t(Use it only from Emacs-Lisp program)."
     (if (YaTeX-main-file-p)
 	(if (interactive-p) (message "I think this is main LaTeX source.") nil)
       (cond
-       ((and (interactive-p) main-file (get-file-buffer main-file))
-	(goto-buffer-window main-file))
-       ((and main-file (YaTeX-switch-to-buffer main-file setbuf)))
+       ((and ;;(interactive-p) 
+	     main-file
+	     (cond ((get-file-buffer main-file)
+		    (cond
+		     (setbuf (set-buffer (get-file-buffer main-file)))
+		     ((get-buffer-window (get-file-buffer main-file))
+		      (select-window
+		       (get-buffer-window (get-file-buffer main-file))))
+		     (t (switch-to-buffer (get-file-buffer main-file)))))
+		   ((file-exists-p main-file)
+		    (funcall ff main-file)))))
+       ;;((and main-file (YaTeX-switch-to-buffer main-file setbuf)))
        ((and main-file
 	     (file-exists-p (setq main-file (concat "../" main-file)))
 	     (y-or-n-p (concat (expand-file-name main-file)
 			       " is main file?:")))
 	(setq YaTeX-parent-file main-file)
-	(YaTeX-switch-to-buffer main-file setbuf))
+	;(YaTeX-switch-to-buffer main-file setbuf)
+	(funcall ff main-file)
+	)
        (t (setq main-file (read-file-name "Enter your main text: " nil nil 1))
 	  (setq YaTeX-parent-file main-file)
-	  (YaTeX-switch-to-buffer main-file setbuf))
+	 ; (YaTeX-switch-to-buffer main-file setbuf))
+	  (funcall ff main-file))
        )))
   nil)
 
